from datetime import datetime

from flask import render_template, Blueprint, redirect, url_for, flash, request
from flask_login import login_required, current_user
from sqlalchemy import and_

from app import Project, db, messages, Shortlisting
from app.forms.project import ProjectForm

bp = Blueprint("staff", __name__, url_prefix="/staff")


@bp.route("/projects")
@login_required
def projects():
    active_projects: list[Project] = (
        Project.query.filter_by(proposer=current_user.username)
        .filter(Project.deleted.is_(None))
        .all()
    )
    return render_template("pages/staff/projects.html", projects=active_projects)


@bp.route("/projects/create", methods=["GET", "POST"])
@login_required
def create_project():
    form = ProjectForm()
    if form.validate_on_submit():
        project = Project(
            title=form.title.data,
            description=form.description.data,
            proposer=current_user.username,
            on_behalf=form.on_behalf.data,
        )
        db.session.add(project)
        db.session.commit()
        return redirect(url_for("staff.projects"))
    return render_template("pages/project-form.html", form=form)


@bp.route("/projects/<project_id>/delete")
@login_required
def delete_project(project_id):
    if project := Project.query.get(project_id):
        project.deleted = datetime.utcnow()
        db.session.commit()
    else:
        flash(messages.PROJECT_NOT_FOUND)
    return redirect(url_for("staff.projects"))


@bp.route("/projects/<project_id>/edit", methods=["GET", "POST"])
@login_required
def edit_project(project_id):
    if project := Project.query.get(project_id):
        form = ProjectForm()
        if form.validate_on_submit():
            project.title = form.title.data
            project.description = form.description.data
            project.on_behalf = form.on_behalf.data
            db.session.commit()
            return redirect(url_for("staff.projects"))
        form = ProjectForm(obj=project)
        return render_template("pages/project-form.html", form=form, project=project)
    flash(messages.PROJECT_NOT_FOUND)
    return redirect(url_for("staff.projects"))


@bp.route("/projects/shortlisted")
@login_required
def projects_ranking():
    projects: list[Shortlisting] = (
        Shortlisting.query.join(Project)
        .filter(Project.proposer == current_user.username)
        .order_by(Shortlisting.staff_ranking)
        .all()
    )
    return render_template("pages/staff/shortlisted-projects.html", projects=projects)


@bp.route("/projects/rankings", methods=["PUT"])
@login_required
def update_ranking():
    shortlisting_ids = request.json
    current_shortlist: list[Shortlisting] = Shortlisting.query.join(Project).filter(
        and_(
            Shortlisting.id.in_(shortlisting_ids),
            Project.proposer == current_user.username,
        )
    )
    for shortlisting in current_shortlist:
        shortlisting.staff_ranking = shortlisting_ids.index(shortlisting.id) + 1
    db.session.commit()
    return "", 204
